﻿using System;
using System.Text.RegularExpressions;
using CoveringArrayView.Model;
using CoveringArrayView.Singleton;

namespace CoveringArrayView.Business
{
    class FunctionParser
    {
        public static bool PatternIsValid(String inFunctionSignature)
        {
            // It should work for any method as follows 
            // 
            //     private void method(type Value)
            //     method(type value)
            //     method(type1 value1, type2 value2)
            //     method(type1 value1, type2 value2){ somecode(); }
            //
            string methodPattern = @"\w+"            // 1st part - Mthod name
                                 + @"\("             // 2nd part - Open Parenthesis
                                 + @"(\w+( )+\w+)+"; // 3rd - First parameter
            methodPattern += @"(,( )*\w+( )+\w+)";   // 4rd - Second or more parameters
            // Make sure there are three parameters if NWay = 3
            if (DataStore.Instance.NWay == 3)
            {
                methodPattern += "{2,}";
            }
            else
            {
                methodPattern += "*";
            }
            methodPattern += @"\)" // 5th = Close Parenthesis
                           + @".*"; // Last - Anything after is ignored

            return Regex.IsMatch(inFunctionSignature, methodPattern);
        }

        public static Function Parse(String inFunctionSignature)
        {
            // Check out if the method pattern isn't even correct
            if (!PatternIsValid(inFunctionSignature)) return null;

            var function = new Function() { OriginalMethod = inFunctionSignature };

            string tmpSignature = inFunctionSignature;
            tmpSignature = SurroundWithSpaces(tmpSignature, '(');
            tmpSignature = SurroundWithSpaces(tmpSignature, ')');
            string[] functionWords = tmpSignature.Split(" ,".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
            function.Name = GetFunctionName(functionWords);
            function.Parameters = GetParameters(functionWords);
            function.MethodSignature = GetMethodSignature(functionWords, function.Parameters.Count);
            return function;
        }

        /// <summary>
        /// This function changes a method from the declaration format
        /// to the usage format with parameters as replaceable tokens:
        /// 
        ///   public void MyMethod(string inString, bool inBool)
        /// 
        /// to
        /// 
        ///   MyMethod({0}, {1})
        /// 
        /// So now stub code can be generated by replacing the tokens.
        /// </summary>
        /// <param name="functionWords"></param>
        /// <param name="inParamCount"></param>
        /// <returns></returns>
        private static string GetMethodSignature(string[] functionWords, int inParamCount)
        {
            string retMthodSignature = string.Empty;
            bool inSideParenthesis = false;
            int paramCount = 0;
            int nextParam = 1;
            int current = 0;
            foreach (var word in functionWords)
            {
                if (word.Equals(")"))
                {
                    inSideParenthesis = false;
                    retMthodSignature += word;
                    continue;
                }

                if (inSideParenthesis)
                {
                    if (current++ == nextParam)
                    {
                        current = 0;
                        retMthodSignature += "{" + paramCount++ + "}";
                        if (paramCount != inParamCount)
                        {
                            retMthodSignature += ", ";
                        }
                        continue;
                    }
                    continue;
                }

                if (word.Equals("("))
                {
                    inSideParenthesis = true;
                    retMthodSignature += word;
                    continue;
                }

                retMthodSignature += word + " ";
            }
            return retMthodSignature;
        }

        private static string GetFunctionName(string[] functionWords)
        {
            for (int i = 0; i < functionWords.Length; i++)
            {
                if (functionWords[i].Contains("("))
                {
                    return functionWords[i - 1];
                }
            }
            return "Invalid function signature: Failed to get the function name.";

        }

        private static Parameters GetParameters(string[] functionWords)
        {
            Parameters parameters = new Parameters();

            int firstParanthesis = -1;
            int lastParanthesis = -1;
            for (int i = 0; i < functionWords.Length; i++)
            {
                if (functionWords[i].Contains("("))
                {
                    firstParanthesis = i;
                }
                if (functionWords[i].Contains(")"))
                {
                    lastParanthesis = i;
                }
            }

            // Return null if the parameters section of the method could not 
            // properly be parsed
            if (firstParanthesis == -1 || lastParanthesis == -1)
            {
                return null;
            }

            // Return an empty dictionary if there are not parameters inside \
            // the function
            if (lastParanthesis - firstParanthesis == 1)
                return parameters;

            // Return the parameters
            for (int i = firstParanthesis + 1; i < lastParanthesis; i += 2)
            {
                if (!parameters.Contains(functionWords[i + 1]))
                    parameters.Add(functionWords[i], functionWords[i + 1]);
            }

            return parameters;
        }

        /// <summary>
        /// This makes sure that the paranthesis have spaces around them.
        /// </summary>
        private static string SurroundWithSpaces(string inFunction, char inChar)
        {
            int pos = inFunction.IndexOf(inChar);
            // If inChar is not found, just return
            if (pos == -1)
                return inFunction;

            if (pos > 0 && inFunction[pos - 1] != ' ')
                inFunction = inFunction.Insert(pos++, " ");
            if (pos < inFunction.Length - 1 && inFunction[pos + 1] != ' ')
                inFunction = inFunction.Insert(pos + 1, " ");

            return inFunction;
        }

    }
}
