﻿using System;
using CoveringArrayAlgorithm;
using CoveringArrayView.Model;
using CoveringArrayView.Singleton;

namespace CoveringArrayView.Business
{
    public class StubCodeBuilder
    {
        /// <summary>
        /// Generates the stub code.
        /// </summary>
        /// <param name="inTestSuite"></param>
        /// <param name="inFunction"></param>
        /// <returns></returns>
        public static string CreateStubTestMethods(TestSuite inTestSuite, Function inFunction)
        {
            string retStubCode = string.Empty;
            retStubCode += "#region Covering Array for " + inFunction.Name + Environment.NewLine;
            retStubCode += "/* " + Environment.NewLine;
            retStubCode += inFunction.OriginalMethod + Environment.NewLine;
            retStubCode += " */" + Environment.NewLine + Environment.NewLine;
            int testNumber = 0;
            foreach (var testCase in inTestSuite.Tests)
            {
                retStubCode += GenerateTestMethod(inFunction, testCase, ref testNumber);
            }
            retStubCode += "#endregion";
            return retStubCode;
        }

        private static string GenerateTestMethod(Function inFunction, TestCase testCase, ref int testNumber)
        {
            // The comment, attribute, and method beginning
            string testMethod = "// Test " + (testNumber++) + Environment.NewLine
                                + "[TestMethod]" + Environment.NewLine
                                + "private void ";

            // The method name amd parentheses
            testMethod = GenerateMethodName(inFunction, testCase, testMethod);

            // The opening squirly backet
            testMethod += "{" + Environment.NewLine;

            // The method code
            if (DataStore.Instance.UseAAA)
            {
                testMethod = GenerateArrangeCode(inFunction, testCase, testMethod);
                testMethod = GenerateActCode(inFunction, testCase, testMethod);
                testMethod = GenerateAssertCode(testMethod);
            }
            else
            {
                GenerateParameterAssignmentCode(inFunction, testCase, testMethod);
            }

            // The close squirly bracket
            testMethod += "}" + Environment.NewLine + Environment.NewLine;
            return testMethod;
        }

        private static string GenerateMethodName(Function inFunction, TestCase testCase, string testMethod)
        {
            testMethod += inFunction.Name + "_";

            int counter = 0;
            for (int i = 0; i < testCase.Count; i++)
            {
                Parameter p = inFunction.Parameters[i];
                string namePart = p.KnownType.TestValues[testCase[i] - counter].Name.Replace(" ", "");
                testMethod += namePart + "_";
                counter += p.KnownType.TestValues.Count;
            }
            testMethod += "Test()" + Environment.NewLine;
            return testMethod;
        }

        private static string GenerateArrangeCode(Function inFunction, TestCase testCase, string testMethod)
        {
            testMethod += "\t// Arrange" + Environment.NewLine;

            testMethod = GenerateParameterAssignmentCode(inFunction, testCase, testMethod);

            testMethod += Environment.NewLine;
            return testMethod;
        }

        private static string GenerateParameterAssignmentCode(Function inFunction, TestCase testCase, string testMethod)
        {
            int counter = 0;
            for (int i = 0; i < testCase.Count; i++)
            {
                Parameter p = inFunction.Parameters[i];
                testMethod += String.Format("\t{0} param{1} = {2};", p.KnownType.ShortTypeName, i + 1,
                                            p.KnownType.TestValues[testCase[i] - counter].GetValueAsCode());
                testMethod += Environment.NewLine;
                counter += p.KnownType.TestValues.Count;
            }
            return testMethod;
        }

        private static string GenerateActCode(Function inFunction, TestCase testCase, string testMethod)
        {
            testMethod += "\t// Act" + Environment.NewLine;
            testMethod += "\t" + inFunction.Name + "(";
            for (int i = 0; i < testCase.Count; i++)
            {
                if (i > 0)
                    testMethod += ", ";
                testMethod += "param" + (i + 1);
            }
            testMethod += ");" + Environment.NewLine;

            testMethod += Environment.NewLine;
            return testMethod;
        }

        private static string GenerateAssertCode(string testMethod)
        {
            testMethod += "\t// Assert" + Environment.NewLine;
            testMethod += "\t// Assert.AreEqual(expected, actual)" + Environment.NewLine;
            return testMethod;
        }
    }
}
